// ==============================================================
//                 ORBITER MODULE: l_1011_bj
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// l_1011_bj.cpp
// Control module for l_1011_bj vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define LAUNCH              3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>
#include <iostream>

#include <string>

#include "OrbiterSoundSDK40.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 50000.0;
const double PB_ISP = 120000.0;
const double PB_MAXMAINTH = 660000;

double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_procl = 0;
double speedbrake = 0;
double power_proc = 0;

double flaps = 0.2;
double Lights = 0;
double switched = 0;
//double aoa;
//double dil = 0;

//double timecounter = 0;


//double i = 0;
//double i1 = 0;
//double i2 = 0;
//double i3 = 0;
//double i4 = 0;
//double i5 = 0;
//double i7 = 0;
//double i9 = 0;
//double i1a = 0;
//double i2a = 0;
//double i3a = 0;
//double i4a = 0;
//double i5a = 0;
//double s1 = 0;
//double s2 = 0;
//double s3 = 0;
//double s4 = 0;
//double s5 = 0;
//double ga = 0;
//double diff = 0.1;
//double k3;
//int k4;
//char k3buffer [33];
//double amet = 0;
//double dee = 0;
//double i13 = 0;
//double i14 = 0;
//double i15 = 0;
//double i16 = 0;
//double i17 = 0;
//double i18 = 0;
//double i19 = 0;
//double secondtime = 0;
double eng_pitch = 22000;

//double hfire = 0;

//double launch = 0;



const VECTOR3 offset = {0,-4,27};
const VECTOR3 ofs = {0,-4.2,0};
const VECTOR3 ofs_sts_sat = {0,-4.2,0};
const VECTOR3 offsetattach = {0,-7.7,0.7};


THRUSTER_HANDLE th_main;
THRUSTER_HANDLE th_retro;


BEACONLIGHTSPEC beacon1;    
BEACONLIGHTSPEC beacon2;  
BEACONLIGHTSPEC beacon3;
BEACONLIGHTSPEC beacon4;

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f155;
MESHHANDLE f158;

ATTACHMENTHANDLE mis_attach;

static TOUCHDOWNVTX tdvtx_gearup[8] = {
	{_V(0,-5.993488,21.77885), 1e6, 1e6, 1, 1},
	{_V(-6.646776,-8.993488,-4.715392), 1e6, 1e6, 1, 1},
	{_V( 6.646776,-8.993488,-4.715392), 1e6, 1e6, 1, 1},
	{_V( 0, 15 ,-30), 1e6, 1e6, 1.5},
	{_V( 0, -2 ,-21), 1e6, 1e6, 1.5},
	{_V( 0,0.4,32.4), 1e6, 1e6, 1.5},
	{_V( 28.5, 1,-14.5), 1e6, 1e6, 1.5},
	{_V(-28.5, 1,-14.5), 1e6, 1e6, 1.5}
	};

static TOUCHDOWNVTX tdvtx_geardown[8] = {
	{_V(0,-9.993488,21.77885), 1e6, 1e6, 0.06, 0.012},
	{_V(-6.646776,-10.293488,-2.715392), 1e6, 1e6, 0.6, 0.12},
	{_V( 6.646776,-10.293488,-2.715392), 1e6, 1e6, 0.6, 0.12},
	{_V( 0, 15 ,-30), 1e6, 1e6, 1.5},
	{_V( 0, -2 ,-21), 1e6, 1e6, 1.5},
	{_V( 0,0.4,32.4), 1e6, 1e6, 1.5},
	{_V( 28.5, 1,-14.5), 1e6, 1e6, 1.5},
	{_V(-28.5, 1,-14.5), 1e6, 1e6, 1.5}
	};

static TOUCHDOWNVTX tdvtx_brakes[8] = {
	{_V(0,-9.993488,21.77885), 1e6, 1e6, 0.5, 0.25},
	{_V(-6.646776,-10.293488,-2.715392), 1e6, 1e6, 3, 1.5},
	{_V( 6.646776,-10.293488,-2.715392), 1e6, 1e6, 3, 1.5},
	{_V( 0, 15 ,-30), 1e6, 1e6, 1.5},
	{_V( 0, -2 ,-21), 1e6, 1e6, 1.5},
	{_V( 0, 0.4,32.4), 1e6, 1e6, 1.5},
	{_V( 28.5, 1,-14.5), 1e6, 1e6, 1.5},
	{_V(-28.5, 1,-14.5), 1e6, 1e6, 1.5}
	};

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	//static const double step = RAD*15.0;
	//static const double istep = 1.0/step;
	//static const int nabsc = 25;
	//static const double CL[nabsc] = {0.3, 0.7, 0.9, 0.9, 0.7, 0.5, 0, -0.5, -0.7, -0.9,  -0.9,  -0.5, 0.2, 0.7, 0.9, 0.9, 0.7, 0.5, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	//static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	//// lift and moment coefficients from -180 to 180 in 15 degree steps.
	//// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	//aoa += PI;
	//int idx = max (0, min (23, (int)(aoa*istep)));
	//double d = aoa*istep - idx;
	//*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	//*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	//*cd = 0.01 + oapiGetInducedDrag (*cl, 2.266, 0.6);

	int i;
	const int nabsc = 9;
	//static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -12*RAD,  0*RAD, 3*RAD, 30*RAD, 60*RAD, 180*RAD};
	//static const double CL[nabsc]  = {       0,      0,     0.4,    1.2,    0.8,     0,      0,      0,       0};
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -1*RAD,  2*RAD, 5*RAD, 10*RAD, 60*RAD, 180*RAD};
	static const double CL[nabsc]  = {       0,      0,      0,      0,    0.3,    1.0,    1.5,      0,       0};
	static const double CM[nabsc]  = {       0,    0.2,    0.1,      0,      0,    -0.1,   -0.2,      0,       0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.04 + 0.2*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + 1*(oapiGetWaveDrag (M, 0.85, 1.0, 1.1, 0.04));
	*cl = 1.75*(*cl);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	/*static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, +0.2, +0.3, +0.2, 0,-0.2,-0.3,-0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);*/

	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.75,      0,   -0.75,  +0.75,     0,   -0.75,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.6*(0.01 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04));
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class l_1011_bj: public VESSEL3 {
public:
	l_1011_bj (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
	void clbkPreStep(double SimT, double SimDT, double MJD);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
    bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
    //void Meshsetup(void);

   
    UINT anim_wheel;
    UINT anim_gear;
    
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    
    UINT anim_flaps;

    UINT anim_power;

    
    bool clbkLoadVC (int id);
    
    int MySoundID;	
	int brakes, brakes_hud;
	int wheel_status, gear_status, tdv_status, flaps_status;
	double wheel_proc;

private:
	SpotLight *slcb1_light;
	SpotLight *slcb2_light;
};

l_1011_bj::l_1011_bj (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-L1011-P");
    f151                 = oapiLoadMeshGlobal ("K33\\K-L1011-P-Cpit");
    f155                 = oapiLoadMeshGlobal ("K33\\K-L1011-P-Cpit_n");
    f158                 = oapiLoadMeshGlobal ("K33\\K-L1011-Attach");
    

	
	DefineAnimations ();
}


void l_1011_bj::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL(GetHandle());

RequestLoadVesselWave(MySoundID,ENG,"Sound\\Vessel\\K-L1011-P\\K-L1011-P-Eng.wav",BOTHVIEW_FADED_FAR);
RequestLoadVesselWave(MySoundID,GEAR,"Sound\\Vessel\\K-L1011-P\\K-L1011-P-Gear.wav",BOTHVIEW_FADED_FAR);
RequestLoadVesselWave(MySoundID,LAUNCH,"Sound\\Vessel\\K-L1011-P\\K-L1011-P-Launch.wav",BOTHVIEW_FADED_FAR);

SoundOptionOnOff(MySoundID,PLAYMAINTHRUST,FALSE);
SoundOptionOnOff(MySoundID,PLAYRETROTHRUST,FALSE);
SoundOptionOnOff(MySoundID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
SoundOptionOnOff(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);
SoundOptionOnOff(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);
SoundOptionOnOff(MySoundID,PLAYRADIOATC,FALSE);
SoundOptionOnOff(MySoundID,DISPLAYTIMER,FALSE);

// add payload mass
double pld_mass = 0;
OBJHANDLE hPayload = GetAttachmentStatus(mis_attach);
if (hPayload)pld_mass = oapiGetMass(hPayload);
SetEmptyMass(110000 + pld_mass);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void l_1011_bj::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;

	brakes = 0;
	brakes_hud = 0;
	wheel_status = 0;
	gear_status = 0;
	tdv_status = 0;
	flaps_status = 0;
	wheel_proc = 0;

	// physical specs
	SetSize (14);
	SetEmptyMass (110000);
	SetCW (0.1, 0.3, 0.6, 0.9);
	SetCrossSections (_V(500,900,100));
	SetRotDrag (_V(3.9,1.3,1.3));
	SetPMI (_V(230,240,50));

	SetCameraOffset (_V(-1,-3.5,26));
	
	//SetTouchdownPoints (_V(0,-9.993488,21.77885), _V(-6.646776,-9.993488,-4.715392), _V(6.646776,-9.993488,-4.715392));
	SetTouchdownPoints ( tdvtx_geardown, 8);

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,8), VLiftCoeff, 12, 330, 9);
	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-24), VLiftCoeff, 10, 110, 9);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,5,-25), HLiftCoeff, 10,  50, 5);


	CreateControlSurface3 (AIRCTRL_ELEVATOR,     40, 1.5, _V(   0,0,-25), AIRCTRL_AXIS_XPOS,1,anim_elevator);
	CreateControlSurface3 (AIRCTRL_RUDDER,       20, 1.0, _V(   0,5,-25), AIRCTRL_AXIS_YPOS,1,anim_rudder);
	CreateControlSurface3 (AIRCTRL_AILERON,      10, 0.5, _V( 25,0,-5), AIRCTRL_AXIS_XPOS,1,anim_raileron);
	CreateControlSurface3 (AIRCTRL_AILERON,      10, 0.5, _V(-25,0,-5), AIRCTRL_AXIS_XNEG,1,anim_laileron);
	CreateControlSurface3 (AIRCTRL_ELEVATORTRIM, 10, 0.5, _V(  0,0,-25), AIRCTRL_AXIS_XPOS,0,anim_elevator);
   // CreateControlSurface3 (AIRCTRL_FLAP,         40, 1.0, _V( 0,0,-4), AIRCTRL_AXIS_XPOS,0,anim_flaps);
	CreateControlSurface3 (AIRCTRL_FLAP,         40, 1.0, _V( 0,0,-4), AIRCTRL_AXIS_XPOS,0);
	
	//CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	//CreateVariableDragElement (&gear_proc, 0.8, _V(0, -1, 0));

	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	//SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-F-15A-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.8, 565, 119, 0.019, 0.121, 1.51, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0.2,-2), _V(0,0,1), PB_MAXMAINTH, hpr,PB_ISP);
	th_retro = CreateThruster (_V(0,0.2,-2), _V(0,0,-1), 0.5*PB_MAXMAINTH, hpr,PB_ISP);
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);
	
	
	
	//AddExhaust (th_main, 5.4, 0.55, _V(-0.55,-0.555,-6), _V(0,0,-1) , tex_main);
	//AddExhaust (th_main, 5.4, 0.55, _V(0.55,-0.555,-6), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(-12.87069,-7.894434,-2.180953), &contrail_main);
    AddExhaustStream (th_main, _V(12.87069,-7.894434,-2.180953), &contrail_main);
    AddExhaustStream (th_main, _V(0,-2.573306,-32.46801), &contrail_main);
    
    
    
    
    // Strobe lights
	
	//Blue strobe
	
	static VECTOR3 beacon1pos = {28.44454,-4,-12.7171};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.33);
		beacon1.falloff = (0.7);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe
	
	static VECTOR3 beacon2pos = {-28.44454,-4,-12.7171};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.33);
		beacon2.falloff = (0.7);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);


	// Landing lights
	
	//left
	
	static VECTOR3 beacon3pos = {-5,-6.25,5.6};
	static VECTOR3 beacon3col = {1,1,1};
	
		beacon3.shape = (BEACONSHAPE_STAR);
		beacon3.pos = &beacon3pos;
		beacon3.col = &beacon3col;
		beacon3.size = (0.5);
		beacon3.falloff = (0.7);
		beacon3.period = (0);
		beacon3.duration = (0);
		beacon3.tofs = (0);
		beacon3.active = false;
		
		AddBeacon (&beacon3);
		
		//Right
	
	static VECTOR3 beacon4pos = {5,-6.25,5.6};
	static VECTOR3 beacon4col = {1,1,1};
	
		beacon4.shape = (BEACONSHAPE_STAR);
		beacon4.pos = &beacon4pos;
		beacon4.col = &beacon4col;
		beacon4.size = (0.5);
		beacon4.falloff = (0.7);
		beacon4.period = (0);
		beacon4.duration = (0);
		beacon4.tofs = (0);
		beacon4.active = false;
		
		AddBeacon (&beacon4);
		
		
		/// light sources
	COLOUR4 col_d = {1.1,1.1,0.99,0};
	COLOUR4 col_s = {1.1,1.1,0.99,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	slcb1_light = (SpotLight*)AddSpotLight(_V(-5,-6.25,5.6), _V(0, 0, 1), 2000, 0.75, 0, 5e-5, RAD*40, RAD*60, col_d, col_s, col_a);
	slcb1_light->Activate(false);

	slcb2_light = (SpotLight*)AddSpotLight(_V(5,-6.25,5.6), _V(0, 0, 1), 2000, 0.75, 0, 5e-5, RAD*40, RAD*60, col_d, col_s, col_a);
	slcb2_light->Activate(false);
    
    

	// visual specs
	
	AddMesh (f150);
	
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    
    mis_attach = CreateAttachment (false, ofs+ofs_sts_sat, _V(0,-1,0), _V(0,0,1), "XS", false);
    
    //else SetAttachmentParams (mis_attach, ofs+ofs_sts_sat, _V(0,-1,0), _V(0,0,1));

     AddMesh (f158,&offsetattach);

}

// ==============================================================
// API callback interface
// ==============================================================

void l_1011_bj::clbkPreStep (double SimT, double SimDT, double MJD)
{
//Pegasus Detach

//if (hfire == 1 && launch == 1)
//	{
//	DetachChild (mis_attach);
//	hfire = 2;
//	launch = 2;
//	}

///////// brakes

if (brakes == 1 && gear_proc == 0 && tdv_status != 1)
	{
		SetTouchdownPoints ( tdvtx_brakes, 8);
		brakes_hud = 1;
		tdv_status = 1;
	}
else if(gear_proc > 0 && tdv_status != 2)
	{
		SetTouchdownPoints ( tdvtx_gearup, 8);
		brakes_hud = 0;
		tdv_status = 2;

	}
else  if(gear_proc == 0 && brakes == 0 && tdv_status != 0)
{
	brakes_hud = 0;
	SetTouchdownPoints ( tdvtx_geardown, 8);
	tdv_status = 0;

}
brakes = 0;

if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
///// engine power vs atm pressure
	double atm_prs = GetAtmPressure ();
	double atm_fact = 0;
	if (atm_prs > 20000)
	{
	atm_fact = atm_prs * (280/81.4)+311203.9;
	if (atm_fact > PB_MAXMAINTH)atm_fact = PB_MAXMAINTH;
	else if (atm_fact < 0)atm_fact = 0;
	SetThrusterMax0 (th_main ,(atm_fact));
	}
	else
	{
	atm_fact = atm_prs * (380/20);
	if (atm_fact > PB_MAXMAINTH)atm_fact = PB_MAXMAINTH;
	else if (atm_fact < 0)atm_fact = 0;
	SetThrusterMax0 (th_main ,(atm_fact));
	}

	
///// wheels rot
	VECTOR3 gspd_vct = _V(0,0,0);
	GetGroundspeedVector(FRAME_LOCAL, gspd_vct);
	double rot_rate = gspd_vct.z/(1.6*PI);
	double delta_rot = SimDT*rot_rate;
	int delta_rot_int = delta_rot;
	if (delta_rot > 1)delta_rot = delta_rot - (delta_rot_int);
	else if (delta_rot < -1)delta_rot = delta_rot - (delta_rot_int);
	wheel_proc = wheel_proc + delta_rot;
	int wheel_proc_int = wheel_proc;
	if (wheel_proc > 1)wheel_proc = wheel_proc - (wheel_proc_int);
	else if (wheel_proc < 0)wheel_proc = wheel_proc - (wheel_proc_int) + 1;

	if ( GroundContact() && gear_status == 0) SetAnimation (anim_wheel, wheel_proc);

// Gear  up down animations


   if( gear_status == 0) gear_proc = 0;
   else if( gear_status == 1)
	{
			gear_proc = gear_proc+(SimDT*0.1);
			if (gear_proc > 1)
			{
				gear_proc = 1;
				gear_status = 2;
			}
	}
   else if( gear_status == 2)gear_proc = 1;
   else if( gear_status == 3)
	{
			gear_proc = gear_proc-(SimDT*0.1);
			if (gear_proc < 0)
			{
				gear_proc = 0;
				gear_status = 0;
			}
	}

	SetAnimation( anim_gear, gear_proc );
	

	 // Flaps


   if( flaps_status == 0) 
   {
	   flaps = flaps;
   }
   else if( flaps_status == 1) 
   {
		flaps = flaps + (SimDT*0.1);
		if(flaps > 1)
		{
			flaps = 1;
			//flaps_status = 2;
		}
	}
   else if( flaps_status == 2) 
	{
	   //flaps = 1;
		flaps = flaps - (SimDT*0.1);
		if(flaps < 0.2)
		{
			flaps = 0.2;
			//flaps_status = 2;
		}
	}
   flaps_status = 0;
   /*else if( flaps_status == 3) 
   {
		flaps = flaps - (SimDT*0.1);
		if(flaps < 0.2)
		{
			flaps = 0.2;
			flaps_status = 0;
		}
	}*/
   

   double ctrl_srf_lvl = (flaps-0.2)*1.25;
   if (ctrl_srf_lvl > 1)ctrl_srf_lvl = 1;
   else  if (ctrl_srf_lvl < 0)ctrl_srf_lvl = 0;

   SetControlSurfaceLevel(AIRCTRL_FLAP,ctrl_srf_lvl);
   SetAnimation( anim_flaps, flaps );

	
	
// Power Guage animation
power_proc = GetThrusterLevel(th_main);	
SetAnimation( anim_power, power_proc );

	
// Engine sound
	
	if (GetThrusterLevel (th_main) > 0) 
	{	
	double pit = ((GetThrusterLevel (th_main)*6000));
	eng_pitch = 13000 + pit;
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
	PlayVesselWave(MySoundID,ENG,LOOP,255,eng_pitch);
	}
	else if (GetThrusterLevel (th_retro) > 0) 
	{	
	double pit = ((GetThrusterLevel (th_retro)*6000));
	eng_pitch = 13000 + pit;
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
	PlayVesselWave(MySoundID,ENG,LOOP,255,eng_pitch);
	}


    if (GetThrusterLevel (th_main) == 0 && GetThrusterLevel (th_retro) == 0) 
	{
	StopVesselWave(MySoundID,ENG);
	}

  //// lights
 if (Lights == 1 && switched == 1)
	{
	 beacon1.active = true;
	 beacon2.active = true;
	 beacon3.active = true;
	 beacon4.active = true;
	 slcb1_light->Activate(true);
	 slcb2_light->Activate(true);
	 switched = 0;
	 Lighting();
	}

if (Lights == 0 && switched == 1)
	{
	beacon1.active = false;
	beacon2.active = false;
	beacon3.active = false;
	beacon4.active = false;
	slcb1_light->Activate(false);
	slcb2_light->Activate(false);
    switched = 0;
    Lighting();
	}



//Ground Steering
if (GroundContact () ) Steering();

}


void l_1011_bj::clbkPostStep (double simt, double simdt, double mjd)
{
    
  
    

}

//Ground Steering


void l_1011_bj::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 1e4 * pos;//* GetAirspeed() / 5.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V(lev,0,0 ), _V( 0,0,30) );
	
	
}

// Lighting


void l_1011_bj::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
	

}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	
}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int l_1011_bj::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_status  == 0)gear_status = 1;
		else if( gear_status  == 3)gear_status = 1;
		else if( gear_status  == 2)gear_status = 3;
		else if( gear_status  == 1)gear_status = 3;

		PlayVesselWave(MySoundID,GEAR,NOLOOP,255);
			return 1;
			
		//	case OAPI_KEY_F: // Flaps
		//
		//if( flaps_status == 0)flaps_status = 1;
		//else if( flaps_status == 1)flaps_status = 3;
		//else if( flaps_status == 2)flaps_status = 3;
		//else if( flaps_status == 3)flaps_status = 1;

		////PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		//
		//	
		//	return 1;

			
		case OAPI_KEY_J: // Launch Pegasus
		
		if( DetachChild (mis_attach) == true)
		{						
		PlayVesselWave(MySoundID,LAUNCH,NOLOOP,255);
		SetEmptyMass(110000);
		return 1;
		}	
			
			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}

int l_1011_bj::clbkConsumeDirectKey(char *kstate)
{
	if (KEYDOWN (kstate, OAPI_KEY_SPACE))
		{
		RESETKEY (kstate, OAPI_KEY_SPACE);
		brakes = 1;
		}
	//return 0;

	if (KEYDOWN (kstate, OAPI_KEY_F))
		{
		//RESETKEY (kstate, OAPI_KEY_F);
		flaps_status = 1;
		}
	//return 0;

	if (KEYMOD_CONTROL (kstate)) 
	{
		if (KEYDOWN (kstate, OAPI_KEY_F))
		{
		//RESETKEY (kstate, OAPI_KEY_F);
		flaps_status = 2;
		}
	//return 0;
	}
 return 0;
}

void l_1011_bj::DefineAnimations(void)
{
   // Front Wheel
   static UINT WGrp1[2] = {35,36};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 2, _V(0,-9.414448,21.77885), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[4] = {37,38,41,43};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 4, _V(0,-9.192282,-2.904032), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[4] = {39,40,42,44};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 4, _V(0,-9.192282,-4.711677), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[6] = {11,12,13,14,35,36};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 6, _V(0,-7.026135,21.61431), _V(1,0,0), (float)(-133*RAD));
   
   //Rear Gear Left
   static UINT GGrp2[10] = {37,38,39,40,47,49,50,51,52,53};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 10, _V(-6.857224,-6.199403,-3.855559), _V(0,0,1), (float)(93*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[10] = {41,42,43,44,48,54,55,56,57,58};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 10, _V(6.857224,-6.199403,-3.855559), _V(0,0,1), (float)(-93*RAD));
   
   //Front Gear Cover left
   static UINT GGrp4[1] = {9};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(-0.4189831,-6.646635,25.73539), _V(0,0,1), (float)(90*RAD));
   
   //Front Gear Cover right
   static UINT GGrp5[1] = {10};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.4189831,-6.646635,25.73539), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Gear Left2
   static UINT GGrp6[10] = {37,38,39,40,47,49,50,51,52,53};
   
   static MGROUP_SCALE gear6 (0,GGrp6, 10, _V(-6.857224,-6.4,-3.855559), _V(0.5,0.5,0.5));
   
   //Rear Gear Right2
   static UINT GGrp7[10] = {41,42,43,44,48,54,55,56,57,58};
   
   static MGROUP_SCALE gear7 (0,GGrp7, 10, _V(6.857224,-6.4,-3.855559), _V(0.5,0.5,0.5));
   
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 1, &gear2);
   AddAnimationComponent  (anim_gear, 0, 1, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 0.75, &gear4);
   AddAnimationComponent  (anim_gear, 0.75, 1, &gear5);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear6);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear7);
   
   
   //Rudder 
   static UINT RGrp1[1] = {59};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 1, _V(0,0.5006016,-27.35149), _V(0,1,-0.4), (float)(-22*RAD));
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   
   //Elevator left
   static UINT EGrp1[1] = {64};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 1, _V(-2.011388,-3.986444,-26.44086), _V(1,0,0.4), (float)(20*RAD));
   
   //Stick Forward
  // static UINT sEGrp1[4] = {82,83,84,85};
   
   //static MGROUP_ROTATE selevator (1,sEGrp1, 4, _V(0,-1.62389,0.6003813), _V(1,0,0), (float)(-20*RAD));
   
   //Elevator Right
   static UINT EGrp2[1] = {65};
   
   static MGROUP_ROTATE elevator2 (0,EGrp2, 1, _V(2.011388,-3.986444,-26.44086), _V(1,0,-0.4), (float)(20*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator2);
   //AddAnimationComponent  (anim_elevator, 0, 1, &selevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {62};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-23.00311,-4.625364,-12.67542), _V(1,0,0.5), (float)(-30*RAD));
   
   //Wheels Turn Right
   static UINT wLAGrp1[1] = {42};
   
   static MGROUP_ROTATE wlaileronr (1,wLAGrp1, 1, _V(1.490325,-0.893803,0.4785985), _V(0,0,1), (float)(-88*RAD));
   
    //Wheels Turn Left
   static UINT wLAGrp2[1] = {41};
   
   static MGROUP_ROTATE wlaileronl (1,wLAGrp2, 1, _V(-1.490325,-0.893803,0.4785985), _V(0,0,1), (float)(-88*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   AddAnimationComponent  (anim_laileron, 0, 1, &wlaileronr);
   AddAnimationComponent  (anim_laileron, 0, 1, &wlaileronl);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {63};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(23.00311,-4.625364,-12.67542), _V(1,0,-0.5), (float)(30*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
  
   
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[7] = {66,67,78,79,80,81,82};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 7, _V(-3.12235,-6.259515,-5.907911), _V(1,0,0.3), (float)(-20*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[7] = {68,69,84,85,86,87,88};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 7, _V(3.12235,-6.259515,-5.907911), _V(1,0,-0.3), (float)(-20*RAD));
   
   anim_flaps = CreateAnimation (0);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);

    // Power Lever Animation
   
   static UINT Pow0[11] = {28,29,30,31,32,33,34,35,36,37,38};
   
   static MGROUP_ROTATE powg (1,Pow0, 11, _V(0,-1.688894,-6.171025E-02), _V(1,0,0), (float)(50*RAD));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);


   
   
}
	

bool l_1011_bj::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(-1.1,-3.6,25));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*44, RAD*44, RAD*44, RAD*44);
	
	//static VCHUDSPEC hud_pilot  = {1, 90,{0,0.13,4.44},0.5};

		
	
	
 return true;

}


//void l_1011_bj::Meshsetup()
//{
//	ClearMeshes();
//	
//	AddMesh (f150);
//    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
//    
//	
//   
//   
//}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new l_1011_bj (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (l_1011_bj*)vessel;
}

bool l_1011_bj::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	//oapiSetDefNavDisplay(0);
	//oapiSetDefRCSDisplay(0);


	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}

	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);

	{
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"G = Landing Gear");
	skp->Text ((10+roxl), (wd+royl), kbuf,strlen(kbuf));
	}
	{
	skp->SetTextColor(0x0066FFFF);
	char kbuf[256];
	if (gear_status == 0)sprintf(kbuf,"Down");
	else if (gear_status == 1)sprintf(kbuf,"Up");
	else if (gear_status == 2)sprintf(kbuf,"Up");
	else if (gear_status == 3)sprintf(kbuf,"Down");
	skp->Text ((300+roxl), (wd+royl), kbuf,strlen(kbuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"SpaceBar = Gear Brakes");
	skp->Text ((10+roxl), (wc+royl), kbuf,strlen(kbuf));
	}
	{
	skp->SetTextColor(0x0066FFFF);
	char kbuf[256];
	if (brakes_hud == 1)sprintf(kbuf,"On");
	else if (brakes_hud == 0)sprintf(kbuf,"Off");
	skp->Text ((300+roxl), (wc+royl), kbuf,strlen(kbuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"F / Ctrl+F = Flaps +/-");
	skp->Text ((10+roxl), (w0+royl), kbuf,strlen(kbuf));
	}

	{
	skp->SetTextColor(0x0066FFFF);
	char kbuf[256];
	sprintf(kbuf,"%d%%", (int)((flaps-0.2)*125));
	/*else if (flaps_status == 1)sprintf(kbuf,"Deploying");
	else if (flaps_status == 2)sprintf(kbuf,"Deployed");
	else if (flaps_status == 3)sprintf(kbuf,"Retracting");*/
	skp->Text ((300+roxl), (w0+royl), kbuf,strlen(kbuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"K = Lights");
	skp->Text ((10+roxl), (w1+royl), kbuf,strlen(kbuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"J = Launch Pegasus");
	skp->Text ((10+roxl), (w2+royl), kbuf,strlen(kbuf));
	}

	{
	double lng, lat, rad = 0;
	GetEquPos(lng, lat, rad);
	OBJHANDLE hEarth = GetGravityRef();
	double ves_alt = GetAltitude();
	VECTOR3 wind_vector = _V(0,0,0);
	double windspeed = 0;
	wind_vector = oapiGetWindVector(hEarth,lng,lat,ves_alt,0,&windspeed);
	double wind_dir = atan2(-wind_vector.x,-wind_vector.z);
	if (wind_dir < 0)wind_dir = wind_dir+PI2;
	int wind_hdg = (int) (DEG*wind_dir);
	if (windspeed == 0)wind_dir = 0;
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"Wind Dir.%d Speed %0.1fm/s", wind_hdg, windspeed);
	skp->Text ((10+roxl), (w3+royl), kbuf,strlen(kbuf));
	//sprintf(oapiDebugString(), "wind_vector %0.1f %0.1f %0.1f",wind_vector.x,wind_vector.y,wind_vector.z);
	}

	{
	
	VECTOR3 grdspd_vector = _V(0,0,0);
	GetGroundspeedVector(FRAME_HORIZON, grdspd_vector);
	double grdspd_dir = atan2(grdspd_vector.x, grdspd_vector.z);
	if(grdspd_dir < 0)grdspd_dir = grdspd_dir+PI2;
	double grdspd_hdg = (grdspd_dir);
	if (length(grdspd_vector) == 0)oapiGetHeading(GetHandle(),&grdspd_hdg);
	skp->SetTextColor(0x0066FF66);
	char kbuf[256];
	sprintf(kbuf,"Groundspeed Hdg. %d", (int)(DEG*grdspd_hdg));
	skp->Text ((10+roxl), (w4+royl), kbuf,strlen(kbuf));
	}

	return true;
}

void l_1011_bj::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	double etl = GetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM);

	// custom parameters
	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%0.1f %d %0.4f %0.4f", Lights, flaps_status, flaps, etl);
	oapiWriteScenario_string (scn, "FLPS", cbuf);


}
void l_1011_bj::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{

	double etl = 0;
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "GEAR", 4)) {
			sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "FLPS", 4)) {
			sscanf (line+4, "%lf%d%lf%lf", &Lights, &flaps_status, &flaps, &etl);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation (anim_gear, gear_proc);
	SetAnimation (anim_flaps, flaps );
	
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM, etl, true);

	if (Lights == 1)switched = 1;
}