// PegasusXL.cpp

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cmath>
#include <stdio.h>
//#include <fstream.h>

// ==========================================================
// Some Orbiter-related parameters
// ==========================================================

#define midpoint(x,y) (((x)+(y))/2)
#define midrad(x,y) (fabs(((x)-(y))/2))

#define COG2(x1,m1,x2,m2)             (((x1)*(m1)+(x2)*(m2))/((m1)+(m2)))
#define COG3(x1,m1,x2,m2,x3,m3)       (((x1)*(m1)+(x2)*(m2)+(x3)*(m3))/((m1)+(m2)+(m3)))
#define COG4(x1,m1,x2,m2,x3,m3,x4,m4) (((x1)*(m1)+(x2)*(m2)+(x3)*(m3)+(x4)*(m4))/((m1)+(m2)+(m3)+(m4)))

#define GREEN RGB(0, 255, 0)
#define YELLOW RGB(255, 255, 0)

const double STAGE1_DIA             = 1.28;
const double STAGE1_EMPTY_MASS = 2886; //Count the interstage towards Stage1
const double STAGE1_PROP_MASS = 15048;
const double STAGE1_MASS=STAGE1_EMPTY_MASS+STAGE1_PROP_MASS;
const double STAGE1_BURN_TIME=68.6;
const double STAGE1_ISP0 = 2846;
const double STAGE1_ISP1 = STAGE1_ISP0*0.8;
//const double STAGE1_ISP0 = 2777;
//const double STAGE1_ISP1 = STAGE1_ISP0*0.61;
const double STAGE1_THRUST = STAGE1_ISP0*STAGE1_PROP_MASS/STAGE1_BURN_TIME;

const double STA_STAGE1_AFT_POINT =   0.47;
const double STA_STAGE1_ENGINE =     STA_STAGE1_AFT_POINT;
const double STA_STAGE1_FWD_MOUNT=   10.81;
const double STA_STAGE1_FWD_POINT=   STA_STAGE1_FWD_MOUNT;

const double STA_STAGE1_FWD_AERO =   STA_STAGE1_FWD_POINT;
const double STA_STAGE1_AFT_AERO =   STA_STAGE1_AFT_POINT;

const double STAGE2_DIA             = STAGE1_DIA;
const double STAGE2_EMPTY_MASS = 416;
const double STAGE2_PROP_MASS = 3915;
const double STAGE2_MASS=STAGE2_EMPTY_MASS+STAGE2_PROP_MASS;
const double STAGE2_BURN_TIME=69.4;
//const double STAGE2_ISP0 = 2838;
const double STAGE2_ISP0 = 2781;
const double STAGE2_ISP1 = STAGE2_ISP0*0.8;
//const double STAGE2_ISP0 = 2749;
//const double STAGE2_ISP1 = STAGE2_ISP0*0.82;
const double STAGE2_THRUST = STAGE2_ISP0*STAGE2_PROP_MASS/STAGE2_BURN_TIME;

const double STA_STAGE2_AFT_POINT =  10.43;
const double STA_STAGE2_ENGINE =     STA_STAGE2_AFT_POINT;
const double STA_STAGE2_AFT_MOUNT=   STA_STAGE1_FWD_MOUNT;
const double STA_STAGE2_MID_MOUNT=   12.99;
const double STA_STAGE2_FWD_MOUNT=   13.54;
const double STA_STAGE2_FWD_POINT=   STA_STAGE2_FWD_MOUNT;

const double STA_STAGE2_FWD_AERO =   STA_STAGE2_MID_MOUNT;
const double STA_STAGE2_AFT_AERO =   STA_STAGE2_AFT_MOUNT;

const double RCS_ISP0 = 2400.0;
const double RCS_ISP1 = 2400.0;
const double RCS_PROP_MASS=6;

const double STAGE3_DIA             = 0.97;
const double STAGE3_EMPTY_MASS = 203;
const double STAGE3_PROP_MASS = 782;
const double STAGE3_MASS=STAGE3_EMPTY_MASS+STAGE3_PROP_MASS;
const double STAGE3_BURN_TIME=75;
const double STAGE3_ISP0 = 2817;
//const double STAGE3_ISP0 = 2777;
const double STAGE3_ISP1 = STAGE3_ISP0*0.8;
//const double STAGE3_ISP1 = STAGE3_ISP0*0.82;
const double STAGE3_THRUST = STAGE3_ISP0*STAGE3_PROP_MASS/STAGE3_BURN_TIME;

const double STA_STAGE3_AFT_POINT=   13.44;
const double STA_STAGE3_ENGINE =     STA_STAGE3_AFT_POINT;
const double STA_STAGE3_AFT_MOUNT=   STA_STAGE2_FWD_POINT;
const double STA_STAGE3_FWD_MOUNT=   14.75;
const double STA_STAGE3_FWD_POINT=   STA_STAGE3_FWD_MOUNT;

const double STA_STAGE3_FWD_AERO =   STA_STAGE3_FWD_MOUNT;
const double STA_STAGE3_AFT_AERO =   STA_STAGE3_AFT_MOUNT;

const double FAIRING_MASS = 157.804;

const double ATT_THRUST = 10;
const double ATT_RAD    = STAGE3_DIA/2;

const double FAIRING_DIA        = STAGE1_DIA;
const double STA_FAIRING_AFT_POINT=   STA_STAGE2_MID_MOUNT;
const double STA_FAIRING_AFT_MOUNT=   STA_STAGE2_MID_MOUNT;
const double STA_FAIRING_FWD_POINT=   17.41;

const double STA_FAIRING_FWD_AERO =   STA_FAIRING_FWD_POINT;
const double STA_FAIRING_AFT_AERO =   STA_FAIRING_AFT_MOUNT;

const double STA_FIN_AFT  =            0.83;
const double STA_FIN_MID  =            1.54;
const double STA_FIN_FWD  =            2.25;
const double FIN_CHORD    =            STA_FIN_FWD-STA_FIN_AFT;
const double FIN_LEN      =            1.53;
const double FIN_RECT_AREA   =         (STA_FIN_MID-STA_FIN_AFT)*FIN_LEN;
const double FIN_TRI_AREA   =          (STA_FIN_FWD-STA_FIN_MID)*FIN_LEN/2;
const double FIN_AREA       =          FIN_RECT_AREA+FIN_TRI_AREA;
const double STA_FIN_RECT_COL   =      midpoint(STA_FIN_MID,STA_FIN_AFT);
const double STA_FIN_TRI_COL    =      (STA_FIN_FWD-STA_FIN_MID)/3+STA_FIN_AFT;
const double STA_FIN_COL    =          COG2(STA_FIN_RECT_COL,FIN_RECT_AREA,STA_FIN_TRI_COL,FIN_TRI_AREA);
const double FIN_ASPECT=FIN_LEN*FIN_LEN/FIN_AREA;

const double STA_WING_AFT   =          5.70;
const double STA_WING_FWD   =          9.70;
const double WING_CHORD     =          STA_WING_FWD-STA_WING_AFT;
const double WING_SPAN      =          6.70;
const double STA_WING_COL   =          WING_CHORD/3+STA_WING_AFT;
const double WING_AREA      =          WING_SPAN*WING_CHORD/2;
const double WING_ASPECT    =          WING_SPAN*WING_SPAN/WING_AREA;

const double STAGE1_CHORD   =          STA_STAGE1_FWD_AERO-STA_STAGE1_AFT_AERO;
const double STAGE1_AREA    =          STAGE1_CHORD*STAGE1_DIA;
const double STAGE1_SPAN    =          STAGE1_DIA;
const double STAGE1_ASPECT  =          STAGE1_SPAN*STAGE1_SPAN/STAGE1_AREA;
const double STA_STAGE1_COL     =          midpoint(STA_STAGE1_FWD_AERO,STA_STAGE1_AFT_AERO);

const double STAGE2_CHORD   =          STA_STAGE2_FWD_AERO-STA_STAGE2_AFT_AERO;
const double STAGE2_AREA    =          STAGE2_CHORD*STAGE1_DIA;
const double STAGE2_SPAN    =          STAGE2_DIA;
const double STAGE2_ASPECT  =          STAGE2_SPAN*STAGE2_SPAN/STAGE2_AREA;
const double STA_STAGE2_COL     =          midpoint(STA_STAGE2_FWD_AERO,STA_STAGE2_AFT_AERO);

const double STAGE3_CHORD   =          STA_STAGE3_FWD_POINT-STA_STAGE3_AFT_MOUNT;
const double STAGE3_AREA    =          STAGE3_CHORD*STAGE3_DIA;
const double STAGE3_SPAN    =          STAGE3_DIA;
const double STAGE3_ASPECT  =          STAGE3_SPAN*STAGE3_SPAN/STAGE3_AREA;
const double STA_STAGE3_COL     =          midpoint(STA_STAGE3_FWD_AERO,STA_STAGE3_AFT_AERO);

const double FAIRING_CHORD   =         STA_FAIRING_FWD_POINT-STA_FAIRING_AFT_POINT;
const double FAIRING_AREA    =         FAIRING_CHORD*FAIRING_DIA;
const double FAIRING_SPAN    =         FAIRING_DIA;
const double FAIRING_ASPECT  =         FAIRING_SPAN*FAIRING_SPAN/FAIRING_AREA;
const double STA_FAIRING_COL    =          midpoint(STA_FAIRING_FWD_AERO,STA_FAIRING_AFT_AERO);

const double BODY_WT = 0.2;
// ==========================================================
// Mesh offsets for various configurations
// ==========================================================

const VECTOR3 OFS_STAGE1  = { 0.0, 0.0,  STA_STAGE1_AFT_POINT};
const VECTOR3 OFS_STAGE2  = { 0.0, 0.0,  STA_STAGE2_AFT_POINT};
const VECTOR3 OFS_STAGE3  = { 0.0, 0.0,  STA_STAGE3_AFT_POINT};
const VECTOR3 OFS_FAIRING = { 0.0, 0.0,  STA_FAIRING_AFT_POINT};
const VECTOR3 OFS_PAYLOAD = { 0.0,0, STA_STAGE3_FWD_POINT-STA_STAGE3_AFT_POINT };

const double STA_STAGE1_COG  =  5.60; //217000 pt MC
const double STA_STAGE2_COG  = 12.08; //202000 pt MC
const double STA_STAGE3_COG  = 14.28; //225000 pt MC
const double STA_FAIRING_COG = 15.32; //232000 pt MC

const double STA_VEH1_COG = COG4(STA_STAGE1_COG,STAGE1_MASS,STA_STAGE2_COG,STAGE2_MASS,STA_STAGE3_COG,STAGE3_MASS,STA_FAIRING_COG,FAIRING_MASS);
const double STA_VEH2_COG = COG2(STA_STAGE2_COG,STAGE2_MASS,STA_STAGE3_COG,STAGE3_MASS);
const double STA_VEH3_COG = STA_STAGE3_COG;
const double STA_ATT =      STA_STAGE3_COG;

const double VEH1_LENGTH =  STA_FAIRING_FWD_POINT-STA_STAGE1_AFT_POINT;
const double VEH2_LENGTH =  STA_FAIRING_FWD_POINT-STA_STAGE2_AFT_POINT;
const double VEH3_LENGTH =  STA_STAGE3_FWD_POINT-STA_STAGE3_AFT_POINT;


float tgt_api;
float tgt_vvely;

// ==========================================================
// Interface for derived vessel class: PegasusXL
// ==========================================================

class PegasusXL: public VESSEL2 {
public:
  PegasusXL (OBJHANDLE hVessel, int flightmodel)
			: VESSEL2 (hVessel, flightmodel) {}
  ~PegasusXL();
 
  void clbkSetClassCaps (FILEHANDLE cfg);
  void clbkSaveState (FILEHANDLE scn);
  void clbkLoadStateEx (FILEHANDLE scn, void *status);
  int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
  void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
  void SetStage1Configuration (void);
  void SetStage2Configuration (void);
  void Setstage3Configuration (void);
  void SetConfiguration (void);
  void AddFairingsMaybe (const VECTOR3 &ofs);
  void AddStage1Visual (const VECTOR3 &ofs);
  void AddStage2Visual (const VECTOR3 &ofs);
  void Addstage3Visual (const VECTOR3 &ofs);
  void AddFairingVisual (const VECTOR3 &ofs, int);
  void SeparateFairings (void);
  void SeparateStage1 (void);
  void SeparateStage2 (void);
  void SeparateStage3 (void);
 // void Timestep (double simt);
  void clbkPreStep (double simt, double simdt, double mjd);
  void Jettison ();
  void UpdateMesh ();
  double CalcEmptyMass();
  int status; //01=Stage1
              // 2=Stage2
              // 3=stage3
  bool bManualSeparate; // flag for user-induced booster or tank separation
  bool bLit1,bLit2,bLit3; // flag for user-induced booster or tank separation
  bool bPayloadAttached;
  bool bFairingAttached;

  VECTOR3 orbiter_ofs;
  VECTOR3 ofs_sts_sat;
  VISHANDLE vis;      // handle for visual - note: we assume that only one visual per object is created!
  MESHHANDLE hStage2Mesh, hStage1Mesh, hstage3Mesh;
  MESHHANDLE hFairingMesh[2]; // mesh handles
  ATTACHMENTHANDLE sat_attach;

  THRUSTER_HANDLE th_rot[16];
  THRUSTER_HANDLE th_group[4];

  AIRFOILHANDLE HFH,HFV;

private:
  void CreateAttControls(PROPELLANT_HANDLE ph, double MaxThr, VECTOR3 AttPoint, double Rad);
  void EnableRotControls();
  void EnableLinControls();

  void Launch(void);
  void DoAutoPilot(void);
  void PitchUp(void);
  void PitchDown(void);
  void PitchStop(void);
  void HeadsUp(void);
  void DoPrograde(void);

  PROPELLANT_HANDLE ph_stage2, ph_stage1, ph_stage3,ph_rcs; // handles for propellant resources
  THRUSTER_HANDLE th_main[1];                // handles for orbiter main engines
  THGROUP_HANDLE thg_main;          // handles for thruster groups

  MGROUP_TRANSFORM *sat_anim, *sat_ref;

  bool reset_sat;
  double jettison_time;
  int timer_sw;
  bool render_cockpit;
  double sim_step;
  double ap_alt_old;
  double vvely_old;

  double timer;

  int autop;
  int stg_sw;
  double initial_pitch;
  double time_to_ign;
  
};

bool SetHdg(void *id, char *str, void *data)
{
	sscanf(str, "%f", &tgt_api);
	return true;
}


//PARTICLESTREAMSPEC srb_contrail = {
//  0, 8.0, 5, 150.0, 0.3, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
//  PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
//  PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
//};
PARTICLESTREAMSPEC srb_exhaust = {
  0, 2, 20, 100.0, 0.15, 0.3, 30, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
  PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.25,
  PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
};

// ==============================================================
// Global (class-wide) parameters
// ==============================================================
// Specialised vessel class PegasusXL
// ==============================================================
// Constructor
void PegasusXL::clbkSetClassCaps (FILEHANDLE cfg)
{
  status          = 1;
  vis             = NULL;

  reset_sat       = false;
  jettison_time   = 0.0;
  render_cockpit  = false;

  timer_sw = 0;
  autop = 0;
  sim_step = 0;
  ap_alt_old = 6371010;
  vvely_old = 0;

  timer = 0;
  stg_sw = 0;
  time_to_ign = 0;

  initial_pitch = 30*RAD;
  tgt_api = 250;
  tgt_vvely = 0;

  // propellant resources
  ph_stage1       = NULL;
  ph_stage2       = NULL;
  ph_stage3       = NULL;
  ph_rcs          = NULL;
  thg_main        = NULL;

  // preload meshes
  hStage1Mesh  = oapiLoadMeshGlobal ("cygnss/Pegasus1");
  hStage2Mesh  = oapiLoadMeshGlobal ("cygnss/Pegasus2");
  hstage3Mesh = oapiLoadMeshGlobal ("cygnss/Pegasus3");
  hFairingMesh[0] = oapiLoadMeshGlobal ("cygnss/PegasusF1");
  hFairingMesh[1] = oapiLoadMeshGlobal ("cygnss/PegasusF2");


  bManualSeparate = false;
  bLit1 = false;
  bLit2 = false;
  bLit3 = false;
  ofs_sts_sat     = OFS_PAYLOAD;
  vis             = NULL;
  sat_attach      = NULL;
  bPayloadAttached=true;
  bFairingAttached=true;

  render_cockpit = false;

  SetCameraOffset(_V(0,1.2,-5));
}

PegasusXL::~PegasusXL () {}

// Set vessel class capabilities from config file
//void PegasusXL::SetClassCaps (FILEHANDLE cfg)
//{
//    render_cockpit = false;
//}

// Read status from scenario file
void PegasusXL::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
  char *line;

  while (oapiReadScenario_nextline (scn, line)) {
    if (!strnicmp (line, "CONF", 4)) {
      sscanf (line+4, "%d", &status);
    } else if (!strnicmp (line, "FAIR", 4)) {
      int a;
      sscanf (line+4, "%d", &a);
      bFairingAttached=(a>0);
    } else if (!strnicmp (line, "AUTP", 4)) {
	  sscanf (line+4, "%d%lf%lf%f", &autop, &jettison_time, &timer, &tgt_api);
	} else {
      ParseScenarioLineEx (line, vs);
     // unrecognised option - pass to Orbiter's generic parser
    }
  }

  switch (status) {
  case 0:
  case 1:
    status=1;
    SetStage1Configuration();
    break;
  case 2:
    SetStage2Configuration();
    break;
  case 3:
    Setstage3Configuration();
    break;
  }

}

// Write status to scenario file
void PegasusXL::clbkSaveState (FILEHANDLE scn)
{
  char cbuf[256];

  // default vessel parameters
  VESSEL2::clbkSaveState (scn);

  // custom parameters
  oapiWriteScenario_int (scn, "CONF", status);

  sprintf (cbuf, "%d %0.1f %0.1f %0.1f", autop, jettison_time, timer, tgt_api);
  oapiWriteScenario_string (scn, "AUTP", cbuf);

  oapiWriteScenario_int (scn, "FAIR", bFairingAttached?1:0);

}

double PegasusXL::CalcEmptyMass() {
  char s[256]="";
  double EmptyMass=0;
  //This code depends on case fallthrough
  switch(status) {
    case 0:
    case 1:
      strcat(s,"Stage1 ");
      EmptyMass+=STAGE1_EMPTY_MASS;
    case 2:
      strcat(s,"Stage2 ");
      EmptyMass+=STAGE2_EMPTY_MASS;
    case 3:
      strcat(s,"stage3 ");
      EmptyMass+=STAGE3_EMPTY_MASS;
    }
    if(bPayloadAttached) {

      OBJHANDLE child;
      if(sat_attach) if(child=GetAttachmentStatus(sat_attach)) {
        strcat(s,"Payload");
        EmptyMass+=oapiGetMass(child);
      }
    }
    if(bFairingAttached) {
    strcat(s,"Fairing ");
      EmptyMass+=FAIRING_MASS;
    }
//    sprintf(oapiDebugString(),"%s empty mass %f payload mass %f",s,EmptyMass,PayloadMass);
    return EmptyMass;
}

void PegasusXL::SetConfiguration() {
  switch(status) {
    case 0:
    case 1:
      status=1;
      SetStage1Configuration();
      break;
    case 2:
      SetStage2Configuration();
      break;
    case 3:
      Setstage3Configuration();
      break;
    }
}

// ==============================================================
// Airfoil coefficient functions (Borrowed from Atlantis)
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
  static const double step = RAD*15.0;
  static const double istep = 1.0/step;
  static const int nabsc = 25;
  static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
  static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
  // lift and moment coefficients from -180 to 180 in 15 degree steps.
  // This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

  aoa += PI;
  int idx = max (0, min (23, (int)(aoa*istep)));
  double d = aoa*istep - idx;
  *cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
  *cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
  *cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

void FinLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
  *cl = sin(2*beta);
  *cm = 0.0;
  *cd = 0.02 + oapiGetInducedDrag (*cl, FIN_ASPECT, 0.8);
}

void BodyLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd) {
  *cl = sin(2*beta)*BODY_WT;
  *cm = 0.0;
  *cd = (0.02 + oapiGetInducedDrag (*cl, 1, 0.6))*BODY_WT;
}



void PegasusXL::SetStage1Configuration (void)
{
//  int i;
//  VECTOR3 ofs;

  // *********************** physical parameters *********************************

  status = 1;
  SetSize(midrad(STA_FAIRING_FWD_POINT,STA_STAGE1_AFT_POINT));
  SetEmptyMass(CalcEmptyMass());
  SetCW(0.5, 1, 2, 2);
  SetCrossSections(_V(24.30,35.23,3.69));
  SetRotDrag(_V(0.7,0.1,0.3));
  SetPMI(_V(14.03,14.17,0.57));
  SetTrimScale (0.05);
  SetTouchdownPoints (_V(0,-5,30), _V(-10,-10,-30), _V(10,0,-30));

  // ************************* propellant specs **********************************

  if (!ph_stage1) ph_stage1 = CreatePropellantResource(STAGE1_PROP_MASS);
  if (!ph_stage2) ph_stage2 = CreatePropellantResource(STAGE2_PROP_MASS);
  if (!ph_stage3) ph_stage3 = CreatePropellantResource(STAGE3_PROP_MASS);
  if (!ph_rcs)    ph_rcs =    CreatePropellantResource(RCS_PROP_MASS);
  SetDefaultPropellantResource (ph_stage1); // display main tank level in generic HUD

  // *********************** thruster definitions ********************************

  th_main[0] = CreateThruster (_V(0,0,STA_STAGE1_ENGINE-STA_VEH1_COG), _V(0,0.0,1), STAGE1_THRUST, ph_stage1, STAGE1_ISP0, STAGE1_ISP1);
  SURFHANDLE exhaust = oapiRegisterExhaustTexture ("exhaust2");
  AddExhaust (th_main[0], 10,1.28/2, exhaust);
  thg_main = CreateThrusterGroup (th_main, 1, THGROUP_MAIN);
  AddExhaustStream(th_main[0], &srb_exhaust);

  // ************************* aerodynamics **************************************

  ClearVariableDragElements ();
  ClearAirfoilDefinitions ();
  /* Vertical lift elements */
  CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_STAGE1_COL -STA_VEH1_COG), BodyLiftCoeff, STAGE1_CHORD,  STAGE1_AREA,  STAGE1_ASPECT);
  CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_STAGE2_COL -STA_VEH1_COG), BodyLiftCoeff, STAGE2_CHORD,  STAGE2_AREA,  STAGE2_ASPECT);
  HFV=CreateAirfoil2(LIFT_VERTICAL, _V( 0,0,STA_FAIRING_COL-STA_VEH1_COG), BodyLiftCoeff, FAIRING_CHORD, FAIRING_AREA, FAIRING_ASPECT);
  CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_WING_COL   -STA_VEH1_COG + 0.95), VLiftCoeff,    WING_CHORD,    WING_AREA,    WING_ASPECT);
  CreateAirfoil(LIFT_VERTICAL, _V( 2,0,STA_FIN_COL    -STA_VEH1_COG), FinLiftCoeff,  FIN_CHORD,     FIN_AREA,     FIN_ASPECT );
  CreateAirfoil(LIFT_VERTICAL, _V(-2,0,STA_FIN_COL    -STA_VEH1_COG), FinLiftCoeff,  FIN_CHORD,     FIN_AREA,     FIN_ASPECT );

  /* Horizontal lift elements */
  CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_STAGE1_COL -STA_VEH1_COG), BodyLiftCoeff, STAGE1_CHORD,  STAGE1_AREA,  STAGE1_ASPECT);
  CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_STAGE2_COL -STA_VEH1_COG), BodyLiftCoeff, STAGE2_CHORD,  STAGE2_AREA,  STAGE2_ASPECT);
  HFH=CreateAirfoil2(LIFT_HORIZONTAL, _V( 0,0,STA_FAIRING_COL-STA_VEH1_COG), BodyLiftCoeff, FAIRING_CHORD, FAIRING_AREA, FAIRING_ASPECT);
  CreateAirfoil(LIFT_HORIZONTAL, _V( 0,2,STA_FIN_COL    -STA_VEH1_COG), FinLiftCoeff,  FIN_CHORD,     FIN_AREA,     FIN_ASPECT );

  CreateControlSurface(AIRCTRL_ELEVATOR, 0.5*FIN_AREA, 2, _V( 0, 0,  STA_FIN_COL-STA_VEH1_COG));
  CreateControlSurface(AIRCTRL_RUDDER,   0.5*FIN_AREA, 0.5, _V( 0, 1,  STA_FIN_COL-STA_VEH1_COG));
  CreateControlSurface(AIRCTRL_AILERON,  0.5*FIN_AREA, 2, _V( 2, 0,  STA_FIN_COL-STA_VEH1_COG));
  CreateControlSurface(AIRCTRL_AILERON,  0.5*FIN_AREA, 2, _V(-2, 0,  STA_FIN_COL-STA_VEH1_COG));

  //CreateControlSurface(AIRCTRL_ELEVATOR, FIN_AREA, 2, _V( 0, 0,  -5));
  //CreateControlSurface(AIRCTRL_RUDDER,   FIN_AREA, 2, _V( 0, 2,  -5));
  //CreateControlSurface(AIRCTRL_AILERON,  FIN_AREA, 2, _V( 2, 0,  -5));
  //CreateControlSurface(AIRCTRL_AILERON,  FIN_AREA, 2, _V(-2, 0,  -5));

  ClearVariableDragElements ();

  SetADCtrlMode (7);
  // ************************ visual parameters **********************************

  ClearMeshes();
  AddStage1Visual (OFS_STAGE1-_V(0,0,STA_VEH1_COG));
  AddStage2Visual (OFS_STAGE2-_V(0,0,STA_VEH1_COG));
  Addstage3Visual (OFS_STAGE3-_V(0,0,STA_VEH1_COG));
  AddFairingsMaybe(_V(0,0,STA_VEH1_COG));
}

void PegasusXL::SetStage2Configuration (void)
{
//  int i;

  // *********************** physical parameters *********************************

  SetSize(midrad(STA_FAIRING_FWD_POINT,STA_STAGE2_AFT_POINT));
  status = 2;
  SetEmptyMass (CalcEmptyMass());
  SetPMI (_V(1.08,1.08,0.18));
  SetCW (0.5, 1, 2, 2);
  SetCrossSections (_V(4.75,4.75,1.28));
  SetCameraOffset(_V(0,2,-3));
  // ************************* aerodynamics **************************************

  ClearVariableDragElements ();
  ClearAirfoilDefinitions ();
  /* Vertical lift elements */
  CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_STAGE2_COL -STA_VEH2_COG-1.75), BodyLiftCoeff, STAGE2_CHORD,  STAGE2_AREA,  STAGE2_ASPECT);
  if(bFairingAttached) {
    CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_FAIRING_COL-STA_VEH2_COG-1.75), BodyLiftCoeff, FAIRING_CHORD, FAIRING_AREA, FAIRING_ASPECT);
  } else {
    CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_STAGE3_COL-STA_VEH2_COG-1.75), BodyLiftCoeff, STAGE3_CHORD, STAGE3_AREA, STAGE3_ASPECT);
  }

  /* Horizontal lift elements */
  CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_STAGE2_COL -STA_VEH2_COG-1.75), BodyLiftCoeff, STAGE2_CHORD,  STAGE2_AREA,  STAGE2_ASPECT);
  if(bFairingAttached) {
    CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_FAIRING_COL-STA_VEH2_COG-1.75), BodyLiftCoeff, FAIRING_CHORD, FAIRING_AREA, FAIRING_ASPECT);
  } else {
    CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_STAGE3_COL-STA_VEH2_COG-1.75), BodyLiftCoeff, STAGE3_CHORD, STAGE3_AREA, STAGE3_ASPECT);
  }

  SetADCtrlMode (0);

  // ************************* particle streams **********************************

 /* PARTICLESTREAMSPEC rps = {
    0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
    PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
  };
  AddReentryStream (&rps);*/

  // ************************* propellant specs **********************************

  if (!ph_stage2)  ph_stage2  = CreatePropellantResource (STAGE2_PROP_MASS);
  if (!ph_stage3)  ph_stage3 = CreatePropellantResource (STAGE3_PROP_MASS);
  if (!ph_rcs)  ph_rcs = CreatePropellantResource (RCS_PROP_MASS);
  SetDefaultPropellantResource (ph_rcs);

  // *********************** thruster definitions ********************************
  DelThruster(th_main[0]);
  th_main[0] = CreateThruster (_V(0,0,STA_STAGE2_ENGINE-STA_VEH2_COG), _V(0,0,1), STAGE2_THRUST, ph_stage2, STAGE2_ISP0, STAGE2_ISP1);
  thg_main = CreateThrusterGroup (th_main, 1, THGROUP_MAIN);
  SURFHANDLE exhaust = oapiRegisterExhaustTexture ("exhaust2");
  AddExhaust (th_main[0], 5, 0.5, exhaust);

  // RCS (Reaction Control System)
  CreateAttControls(ph_rcs,ATT_THRUST,_V(0,0,STA_ATT-STA_VEH2_COG),ATT_RAD);

  // ************************ visual parameters **********************************

  ClearMeshes();
  AddStage2Visual (OFS_STAGE2-_V(0,0,STA_VEH2_COG));
  Addstage3Visual (OFS_STAGE3-_V(0,0,STA_VEH2_COG));
  AddFairingsMaybe(_V(0,0,STA_VEH2_COG));

}

void PegasusXL::Setstage3Configuration (void)
{
//  int i;

  // *********************** physical parameters *********************************
	SetCameraOffset(_V(0,2,-1));
  SetSize(midrad(STA_FAIRING_FWD_POINT,STA_STAGE3_AFT_POINT));
  status = 3;
  SetEmptyMass (CalcEmptyMass());
  SetCW (0.5, 1, 2, 2);
  SetPMI (_V(0.12,0.12,0.11));
  SetCrossSections (_V(1.24,1.24,0.73));

  // ************************* aerodynamics **************************************

  ClearVariableDragElements ();
  ClearAirfoilDefinitions ();
  CreateAirfoil(LIFT_VERTICAL, _V( 0,0,STA_STAGE3_COL-STA_VEH3_COG-1.75), BodyLiftCoeff, STAGE3_CHORD, STAGE3_AREA, STAGE3_ASPECT);
  CreateAirfoil(LIFT_HORIZONTAL, _V( 0,0,STA_STAGE3_COL-STA_VEH3_COG-1.75), BodyLiftCoeff, STAGE3_CHORD, STAGE3_AREA, STAGE3_ASPECT);

  SetADCtrlMode(0);

  // ************************* particle streams **********************************

 /* PARTICLESTREAMSPEC rps = {
    0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
    PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
  };
  AddReentryStream (&rps);*/

  // ************************* propellant specs **********************************

  if (!ph_stage3)  ph_stage3 = CreatePropellantResource (STAGE3_PROP_MASS);
  if (!ph_rcs)     ph_rcs = CreatePropellantResource (RCS_PROP_MASS);
  SetDefaultPropellantResource (ph_rcs);

  // *********************** thruster definitions ********************************
  DelThruster(th_main[0]);
  th_main[0] = CreateThruster (_V(0,0,STA_STAGE3_ENGINE-STA_VEH3_COG), _V(0,0.0,1), STAGE3_THRUST, ph_stage3, STAGE3_ISP0, STAGE3_ISP1);
  thg_main = CreateThrusterGroup (th_main, 1, THGROUP_MAIN);
  SURFHANDLE exhaust = oapiRegisterExhaustTexture ("exhaust2");
  AddExhaust (th_main[0], 5, 0.3, exhaust);

  // RCS (Reaction Control System)
  CreateAttControls(ph_rcs,ATT_THRUST,_V(0,0,STA_ATT-STA_VEH3_COG),ATT_RAD);

  // ************************ visual parameters **********************************

  ClearMeshes();
  Addstage3Visual (OFS_STAGE3-_V(0,0,STA_VEH3_COG));
}

void PegasusXL::CreateAttControls(PROPELLANT_HANDLE ph, double MaxThr, VECTOR3 AttPoint, double Rad)
{
  // we should only require pitch and bank, but yaw is also defined to make KILLROT work
  // no linear attitude modes are defined at this stage
  const int A=0; const int B=4; const int C=8; const int D=12;
  const int forward=0; const int left=1; const int up=1; const int backward=2; const int right=3; const int down=3;
  //Quad A (top)
  th_rot[A+forward ] = CreateThruster (AttPoint+_V(0, Rad,0), _V( 0, 0, 1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Forward
  th_rot[A+left    ] = CreateThruster (AttPoint+_V(0, Rad,0), _V(-1, 0, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Left
  th_rot[A+backward] = CreateThruster (AttPoint+_V(0, Rad,0), _V( 0, 0,-1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Backward
  th_rot[A+right   ] = CreateThruster (AttPoint+_V(0, Rad,0), _V( 1, 0, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Right
  //Quad B (right)
  th_rot[B+forward ] = CreateThruster (AttPoint+_V( Rad,0,0), _V( 0, 0, 1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Forward
  th_rot[B+up      ] = CreateThruster (AttPoint+_V( Rad,0,0), _V( 0, 1, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Up
  th_rot[B+backward] = CreateThruster (AttPoint+_V( Rad,0,0), _V( 0, 0,-1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Backward
  th_rot[B+down    ] = CreateThruster (AttPoint+_V( Rad,0,0), _V( 0,-1, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Down
  //Quad C (bottom)
  th_rot[C+forward ] = CreateThruster (AttPoint+_V(0,-Rad,0), _V( 0, 0, 1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Forward
  th_rot[C+left    ] = CreateThruster (AttPoint+_V(0,-Rad,0), _V(-1, 0, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Left
  th_rot[C+backward] = CreateThruster (AttPoint+_V(0,-Rad,0), _V( 0, 0,-1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Backward
  th_rot[C+right   ] = CreateThruster (AttPoint+_V(0,-Rad,0), _V( 1, 0, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Right
  //Quad D (left)
  th_rot[D+forward ] = CreateThruster (AttPoint+_V(-Rad,0,0), _V( 0, 0, 1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Forward
  th_rot[D+up      ] = CreateThruster (AttPoint+_V(-Rad,0,0), _V( 0, 1, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Up
  th_rot[D+backward] = CreateThruster (AttPoint+_V(-Rad,0,0), _V( 0, 0,-1), MaxThr*2, ph, RCS_ISP0, RCS_ISP1); //Backward
  th_rot[D+down    ] = CreateThruster (AttPoint+_V(-Rad,0,0), _V( 0,-1, 0), MaxThr, ph, RCS_ISP0, RCS_ISP1); //Down

  for(int i=0;i<16;i++) AddExhaust (th_rot[i], 1.0,0.1);

  EnableRotControls();
  EnableLinControls();
}

void PegasusXL::EnableRotControls() {

  const int A=0; const int B=4; const int C=8; const int D=12;
  const int forward=0; const int left=1; const int up=1; const int backward=2; const int right=3; const int down=3;

  th_group[0]=th_rot[A+backward];
  th_group[0]=th_rot[C+forward];
  CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

  th_group[0]=th_rot[A+forward];
  th_group[0]=th_rot[C+backward];
  CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

  th_group[0]=th_rot[A+left];
  th_group[1]=th_rot[B+up];
  th_group[2]=th_rot[C+right];
  th_group[3]=th_rot[D+down];
  CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

  th_group[0]=th_rot[A+right];
  th_group[1]=th_rot[B+down];
  th_group[2]=th_rot[C+left];
  th_group[3]=th_rot[D+up];
  CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

  th_group[0]=th_rot[D+forward];
  th_group[0]=th_rot[B+backward];
  CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

  th_group[0]=th_rot[D+backward];
  th_group[0]=th_rot[B+forward];
  CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);
}

void PegasusXL::EnableLinControls() {

  const int A=0; const int B=4; const int C=8; const int D=12;
  const int forward=0; const int left=1; const int up=1; const int backward=2; const int right=3; const int down=3;

  th_group[0]=th_rot[B+up];
  th_group[1]=th_rot[D+up];
  CreateThrusterGroup (th_group, 2, THGROUP_ATT_UP);

  th_group[0]=th_rot[B+down];
  th_group[1]=th_rot[D+down];
  CreateThrusterGroup (th_group, 2, THGROUP_ATT_DOWN);

  th_group[0]=th_rot[A+forward];
  th_group[1]=th_rot[B+forward];
  th_group[2]=th_rot[C+forward];
  th_group[3]=th_rot[D+forward];
  CreateThrusterGroup (th_group, 4, THGROUP_ATT_FORWARD);

  th_group[0]=th_rot[A+backward];
  th_group[1]=th_rot[B+backward];
  th_group[2]=th_rot[C+backward];
  th_group[3]=th_rot[D+backward];
  CreateThrusterGroup (th_group, 4, THGROUP_ATT_BACK);

  th_group[0]=th_rot[A+left];
  th_group[1]=th_rot[C+left];
  CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

  th_group[0]=th_rot[A+right];
  th_group[1]=th_rot[C+right];
  CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);
}

void PegasusXL::AddFairingsMaybe(const VECTOR3 &a) {
  if(bFairingAttached) {
      AddFairingVisual(OFS_FAIRING-a,0);
      AddFairingVisual(OFS_FAIRING-a,1);
  }
}

void PegasusXL::AddStage1Visual (const VECTOR3 &ofs)
{
  UINT meshidx;
  meshidx = AddMesh (hStage1Mesh, &ofs);
  SetMeshVisibilityMode (meshidx, MESHVIS_ALWAYS);
}

void PegasusXL::AddStage2Visual (const VECTOR3 &ofs)
{
  UINT meshidx;
  meshidx = AddMesh (hStage2Mesh, &ofs);
  SetMeshVisibilityMode (meshidx, MESHVIS_ALWAYS);
}

void PegasusXL::Addstage3Visual (const VECTOR3 &ofs)
{
  orbiter_ofs = ofs;

  UINT meshidx;
  meshidx = AddMesh (hstage3Mesh, &ofs);
  SetMeshVisibilityMode (meshidx, MESHVIS_ALWAYS);

  if (!sat_attach) sat_attach = CreateAttachment (false, ofs+ofs_sts_sat, _V(0,0,1), _V(0,1,0), "X");
  else SetAttachmentParams (sat_attach, ofs+ofs_sts_sat, _V(0,0,1), _V(0,1,0));
}

void PegasusXL::AddFairingVisual (const VECTOR3 &ofs,int FairingIdx)
{
  UINT meshidx;
  meshidx = AddMesh (hFairingMesh[FairingIdx], &ofs);
  SetMeshVisibilityMode (meshidx, MESHVIS_ALWAYS);
}

void PegasusXL::SeparateFairings ()
{
  VESSELSTATUS vs;
  GetStatus (vs);
  VECTOR3 ofs = OFS_FAIRING-_V(0,0,STA_STAGE2_COG);
  VECTOR3 rofs1,rofs2, rvel = vs.rvel;
  VECTOR3 lvrot = vs.vrot;
  VECTOR3 vel1 = {-2,0,0};
  VECTOR3 vel2 = {2,0,0};
  Local2Rel (ofs, vs.rpos);
  GlobalRot (vel1, rofs1);
  GlobalRot (vel2, rofs2);
  vs.rvel.x = rvel.x+rofs1.x;
  vs.rvel.y = rvel.y+rofs1.y;
  vs.rvel.z = rvel.z+rofs1.z;
  vs.vrot.y = lvrot.y + 0.2;
  vs.status = 0;
  char name[256];
  strcpy (name, GetName()); strcat (name, "-Fairing1");
  oapiCreateVessel (name, "Vessels/cygnss/PegasusXL_Fairing1", vs);
  vs.rvel.x = rvel.x+rofs2.x;
  vs.rvel.y = rvel.y+rofs2.y;
  vs.rvel.z = rvel.z+rofs2.z;
  vs.vrot.y = lvrot.y - 0.2;
  name[strlen(name)-1] = '2';
  oapiCreateVessel (name, "Vessels/cygnss/PegasusXL_Fairing2", vs);

  // reconfigure
  bFairingAttached=false;
  SetConfiguration();
}

void PegasusXL::SeparateStage1 (void)
{
  // Create Tank as individual object
  VESSELSTATUS vs;
  GetStatus (vs);
  VECTOR3 ofs = _V(0,0,STA_STAGE2_AFT_MOUNT-STA_STAGE2_COG-STA_STAGE1_FWD_MOUNT+STA_STAGE1_AFT_POINT);
  Local2Rel (ofs, vs.rpos);
  VECTOR3 vofsr = {0,0,-0.3};
  VECTOR3 vofsg;
  GlobalRot(vofsr, vofsg);
  vs.rvel.x = vs.rvel.x + vofsg.x;
  vs.rvel.y = vs.rvel.y + vofsg.y;
  vs.rvel.z = vs.rvel.z + vofsg.z;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  char name[256];
  strcpy (name, GetName()); strcat (name, "-Stage1");
  oapiCreateVessel (name, "Vessels/cygnss/PegasusXL_Stage1", vs);

  // Remove Tank from shuttle instance
  DelPropellantResource (ph_stage1);

  // main engines are done
  double ThrLevel=GetThrusterLevel(th_main[0]);
  DelThrusterGroup (thg_main, THGROUP_MAIN, true);

  // reconfigure
  SetStage2Configuration ();
}

void PegasusXL::SeparateStage2(void)
{
  // Create Tank as individual object
  VESSELSTATUS vs;
  GetStatus (vs);
  VECTOR3 ofs = _V(0,0,STA_STAGE3_AFT_MOUNT-STA_STAGE3_COG-STA_STAGE2_FWD_MOUNT+STA_STAGE2_AFT_POINT);
  Local2Rel (ofs, vs.rpos);
  VECTOR3 vofsr = {0,0,-0.3};
  VECTOR3 vofsg;
  GlobalRot(vofsr, vofsg);
  vs.rvel.x = vs.rvel.x + vofsg.x;
  vs.rvel.y = vs.rvel.y + vofsg.y;
  vs.rvel.z = vs.rvel.z + vofsg.z;
  vs.status = 0;
  char name[256];
  strcpy (name, GetName()); strcat (name, "-Stage2");
  oapiCreateVessel (name, "Vessels/cygnss/PegasusXL_Stage2", vs);

  DelPropellantResource (ph_stage2);
  DelThrusterGroup (thg_main, THGROUP_MAIN, true);

  // clear launch attitude control system
  DelThrusterGroup (THGROUP_ATT_PITCHUP, true);
  DelThrusterGroup (THGROUP_ATT_PITCHDOWN, true);
  DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
  DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
  DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
  DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

  // reconfigure
  Setstage3Configuration ();
}

void PegasusXL::SeparateStage3() {
  if (GetAttachmentStatus (sat_attach)) {
    OBJHANDLE child=GetAttachmentStatus(sat_attach);
    DetachChild (sat_attach, 0.1);
//    oapiSetFocusObject(child);
    SetEmptyMass (CalcEmptyMass());
  }
}

void PegasusXL::clbkPreStep (double simt, double simdt, double mjd)
{
	sim_step = simdt;
//  int i;
//  OBJHANDLE hvessel;
  SetEmptyMass (CalcEmptyMass());

ATTACHMENTHANDLE sg_attach = GetAttachmentHandle(true,0);
OBJHANDLE hStargazer = GetAttachmentStatus(sg_attach);
if (hStargazer == NULL)
{
jettison_time = jettison_time + simdt;
}
 
if (autop == 1) DoAutoPilot();
double th_lvl = GetThrusterLevel(th_main[0]);

  switch (status) {
  case 1: // Stage 1
	
    if(th_lvl>0) bLit1=true;
    if(bLit1) SetThrusterLevel(th_main[0],th_lvl);
    if(GetPropellantMass(ph_stage1)>1) {
    } else {
    if(bLit1) {
        SetDefaultPropellantResource(ph_rcs);
        SetThrusterLevel(th_main[0],0);
        bLit1=false;
      }
    }
    if (bManualSeparate) {
      SeparateStage1();
      bManualSeparate = false;
    }
    break;
  case 2: // Stage 2
    if(GetThrusterLevel(th_main[0])>0) {
    if(!bLit2) {
        SetADCtrlMode(3);
        bLit2=true;
      }
    }
    if(bLit2) {
      SetThrusterLevel(th_main[0],1);
      SetDefaultPropellantResource (ph_stage2);
    }
    if(GetPropellantMass(ph_stage2)>1) {
      double tempP = 0.0;
      double tempY = 0.0;
      tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);
      tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);
      if(autop == 0)SetThrusterDir(th_main[0],_V(0.00524*tempY,0.00524*tempP,1));
    } else {
      if(bLit2) {
		  stg_sw = 1;
        SetDefaultPropellantResource(ph_rcs);
        SetThrusterLevel(th_main[0],0);
        SetADCtrlMode(0);
        bLit2=false;
      }
    }
    if (!bLit2 && bManualSeparate) {
      SeparateStage2();
      bManualSeparate = false;
    }
    break;
  case 3: // Stage3
    if(GetThrusterLevel(th_main[0])>0) {
    if(!bLit3) {
        SetADCtrlMode(3);
        SetDefaultPropellantResource(ph_stage3);
      }
    bLit3=true;
    }
    if(bLit3) SetThrusterLevel(th_main[0],1);
    if(GetPropellantMass(ph_stage3)>1) {
      double tempP = 0.0;
      double tempY = 0.0;
      tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);
      tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);
    if(autop == 0)SetThrusterDir(th_main,_V(0.00524*tempY,0.00524*tempP,1));
    } else {
    if(bLit3) {
        SetDefaultPropellantResource (ph_rcs);
        SetThrusterLevel(th_main[0],0);
        SetADCtrlMode(0);
        bLit3=false;
      }
    }
    if (!bLit3 && bManualSeparate) {
      SeparateStage3();
      bManualSeparate = false;
    }

    break;
  }

}

void PegasusXL::Jettison ()
{
  switch (status) {
  case 2:
    SeparateStage1();
    break;
  case 3:               // abandon tank
    SeparateStage2();
    break;
  }
}

double linterp(double x1, double y1, double x2, double y2, double x) {
  double deltax=(x2-x1);
  double deltay=(y2-y1);
  double frac=(x-x1)/deltax;
  return y1+deltay*frac;
}

// ==============================================================
// DLL entry point
// ==============================================================

//BOOL WINAPI DllMain (HINSTANCE hModule,
//           DWORD ul_reason_for_call,
 //          LPVOID lpReserved)
//{
//  switch (ul_reason_for_call) {
//  case DLL_PROCESS_ATTACH:
//    break;
//  case DLL_PROCESS_DETACH:
 //   break;
 // }
 // return TRUE;
//}

// ==============================================================
// API interface
// ==============================================================

// Vessel initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
  return new PegasusXL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
  if (vessel) delete (PegasusXL*)vessel;
}


DLLCLBK void ovcFocusChanged (VESSEL *vessel, bool getfocus, OBJHANDLE newv, OBJHANDLE oldv)
{
  if (getfocus) {
    oapiDisableMFDMode (MFD_LANDING);
  }
}

//DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
//{
//  ((PegasusXL*)vessel)->Timestep (simt);
//}

DLLCLBK void ovcVisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
  if (refcount > 1) return; // we don't allow more than one visual per object
  PegasusXL *sts = (PegasusXL*)vessel;
  sts->vis = vis;


}

DLLCLBK void ovcVisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
  PegasusXL *sts = (PegasusXL*)vessel;
  sts->vis = NULL;
}



// Keyboard interface handler (buffered key events)
//DLLCLBK int ovcConsumeBufferedKey (VESSEL *vessel, DWORD key, bool down, char *kstate)
int PegasusXL::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
  if (!down) return 0; // only process keydown events

  //PegasusXL *sts = (PegasusXL*)vessel;

  if (KEYMOD_SHIFT (kstate)) {

  } else { // unmodified keys
    switch (key) {
    case OAPI_KEY_J:  // Jettison
      //if(!(sts->bFairingAttached && sts->status==2)) sts->bManualSeparate = true;
	  if(!(bFairingAttached && status==2))bManualSeparate = true;
      return 1;
    case OAPI_KEY_K:  // Jettison Fairing
      //if(sts->status==2 && sts->bFairingAttached) sts->SeparateFairings();
	  if(status==2 && bFairingAttached)SeparateFairings();
      return 1;

	case OAPI_KEY_N:  // launch
		Launch();
	return 1;

	case OAPI_KEY_M:  // autopilot switch
		if (autop == 0)autop = 1;
		else if (autop == 1)
		{
			autop = 0;
			SetControlSurfaceLevel(AIRCTRL_ELEVATOR,0,true);
			SetControlSurfaceLevel(AIRCTRL_AILERON,0,true);
			SetThrusterDir(th_main[0], _V(0,0,1));
			SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
			SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
		}
		return 1;
	case OAPI_KEY_B: // dialog box
		oapiOpenInputBox("Stg.2 Apogee Tgt.(km)", SetHdg, 0, 25, 0);
		return 1;

    }
  }
  return 0;
}

void PegasusXL::Launch (void)
{
	ATTACHMENTHANDLE sg_attach = GetAttachmentHandle(true,0);

	OBJHANDLE hStargazer = GetAttachmentStatus(sg_attach);
	if (hStargazer == NULL) return;
	else
	{
		VESSEL *v = oapiGetVesselInterface (hStargazer);
		int ac = v->AttachmentCount(false);
		ATTACHMENTHANDLE hAttSG;
		for (int j = 0; j < ac; j++)
		{
		hAttSG = v->GetAttachmentHandle(false,j);
		OBJHANDLE me = v->GetAttachmentStatus(hAttSG);
		if (GetHandle() == me)
			{	
			v->DetachChild(hAttSG);
			return;
			}
		}
	}
}

void PegasusXL::DoAutoPilot (void)
{
	VECTOR3 avrot;
	GetAngularVel(avrot);
	double pitch = GetPitch();
	double bank_angle = GetBank();

	

	if (status == 1 && GetPropellantMass(ph_stage1) > 1)
	{
	if (GetThrusterLevel(th_main[0]) < 1 && jettison_time > 5) SetThrusterLevel(th_main[0], 1);

	VECTOR3 acrot;
	GetAngularAcc(acrot);
	double elev_level = GetControlSurfaceLevel(AIRCTRL_ELEVATOR);
	double att_angle = GetAOA();
	double aln_level = GetControlSurfaceLevel(AIRCTRL_AILERON);
	
	//sprintf(oapiDebugString(),"bank %0.2f xrot %0.2f aln %0.2f",bank_angle*DEG, avrot.z*DEG, aln_level);

	initial_pitch = (((tgt_api - 250)/50)+35);
	if (initial_pitch < 35)initial_pitch = 35;
	else if (initial_pitch > 45)initial_pitch = 45;
	initial_pitch = initial_pitch*RAD;

	double max_avrot = 5*RAD;

	double tgt_pitch = 0*RAD;
	if (jettison_time > 37)
	{	
	max_avrot = 2*RAD;
	tgt_pitch = pitch-att_angle;
	}
	else if (jettison_time > 5) tgt_pitch = initial_pitch;

	//sprintf(oapiDebugString(),"jettison_time  %0.1f",jettison_time);

	

	double pf = avrot.x/(5*RAD);
	if (pf < 0) pf = -pf;
	if (pf < 0.5) pf = 0.5;

	double bf = avrot.z/(5*RAD);
	if (bf < 0) bf = -bf;

	double dt = sim_step*8;
	if((tgt_pitch-pitch) < 0.25*RAD && (tgt_pitch-pitch) > -0.25*RAD)dt = sim_step*2*pf;

	double dp = sim_step*2;
	if(bank_angle < 0.5*RAD && bank_angle > -0.5*RAD) dp = sim_step*1*bf;

	if ((tgt_pitch-pitch) > 5*RAD)
	{
		if (avrot.x < 0)elev_level = elev_level + dt;
		else if (avrot.x < max_avrot)elev_level = elev_level + dt;
		else if(avrot.x > max_avrot)elev_level = elev_level - dt;

		if (elev_level > 1)elev_level = 1;
		else if (elev_level < -1)elev_level = -1;
		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,elev_level,true);

	}
	else if ((tgt_pitch-pitch) < 5*RAD && (tgt_pitch-pitch) > 0)
	{
		

		
		double timetostop = 2*(tgt_pitch-pitch)/avrot.x;
		double req_acc = -avrot.x/timetostop;
		if (timetostop > 2)elev_level = elev_level + dt;
		else if (avrot.x < 0)elev_level = elev_level + dt;
		else if (acrot.x > req_acc)elev_level = elev_level - dt;
		else if (acrot.x < req_acc)elev_level = elev_level + dt;

		if (elev_level > 1)elev_level = 1;
		else if (elev_level < -1)elev_level = -1;
		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,elev_level,true);
	}
	else if ((tgt_pitch-pitch) < 0 && (tgt_pitch-pitch) > -5*RAD)
	{
		

		
		double timetostop = 2*(tgt_pitch-pitch)/avrot.x;
		double req_acc = -avrot.x/timetostop;

		if (timetostop > 2)elev_level = elev_level - dt;
		else if (avrot.x > 0)elev_level = elev_level - dt;
		else if (acrot.x < req_acc)elev_level = elev_level + dt;
		else if (acrot.x > req_acc)elev_level = elev_level - dt;

		if (elev_level > 1)elev_level = 1;
		else if (elev_level < -1)elev_level = -1;
		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,elev_level,true);
	}
	else if ((tgt_pitch-pitch) < -5*RAD)
	{
		if (avrot.x > 0)elev_level = elev_level - dt;
		else if (avrot.x < -max_avrot)elev_level = elev_level + dt;
		else if(avrot.x > -max_avrot)elev_level = elev_level - dt;

		if (elev_level > 1)elev_level = 1;
		else if (elev_level < -1)elev_level = -1;
		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,elev_level,true);

	}

	//////// bank
	if (bank_angle > 5*RAD)
		{
		if (avrot.z < 0)aln_level = aln_level + dp;
		else if (avrot.z < max_avrot)aln_level = aln_level + dp;
		else if(avrot.z > max_avrot)aln_level = aln_level - dp;

		if (aln_level > 1)aln_level = 1;
		else if (aln_level < -1)aln_level = -1;
		SetControlSurfaceLevel(AIRCTRL_AILERON,aln_level,true);
		}
	else if (bank_angle < 5*RAD && bank_angle > 0)
		{
		double timetostop = 2*bank_angle/avrot.z;
		double req_acc = avrot.z/timetostop;
		if (avrot.z < 0)aln_level = aln_level + dp;
		else if (acrot.z > req_acc)aln_level = aln_level - dp;
		else if (acrot.z < req_acc)aln_level = aln_level + dp;

		if (aln_level > 1)aln_level = 1;
		else if (aln_level < -1)aln_level = -1;
		SetControlSurfaceLevel(AIRCTRL_AILERON,aln_level,true);
		}
	else if (bank_angle < 0 && bank_angle > -5*RAD)
		{
		double timetostop = 2*bank_angle/avrot.z;
		double req_acc = avrot.z/timetostop;
		if (avrot.z > 0)aln_level = aln_level - dp;
		else if (acrot.z < req_acc)aln_level = aln_level + dp;
		else if (acrot.z > req_acc)aln_level = aln_level - dp;

		if (aln_level > 1)aln_level = 1;
		else if (aln_level < -1)aln_level = -1;
		SetControlSurfaceLevel(AIRCTRL_AILERON,aln_level,true);
		}
	else if (bank_angle < -5*RAD)
		{
		if (avrot.z > 0)aln_level = aln_level - dp;
		else if (avrot.z > -max_avrot)aln_level = aln_level - dp;
		else if(avrot.z < -max_avrot)aln_level = aln_level + dp;

		if (aln_level > 1)aln_level = 1;
		else if (aln_level < -1)aln_level = -1;
		SetControlSurfaceLevel(AIRCTRL_AILERON,aln_level,true);
		}
	if (GetThrusterLevel(th_main[0]) > 0)
		{
	double max_th_lvl = 50/(GetThrusterLevel(th_main[0])*STAGE1_THRUST/GetMass()); 
	if (max_th_lvl > 1)max_th_lvl = 1;
	else if (max_th_lvl < 0)max_th_lvl = 0;
	SetThrusterLevel(th_main[0], max_th_lvl);
		}
	}
	else if (status == 1 && GetPropellantMass(ph_stage1) < 1)
	{
		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,0,true);
		SetControlSurfaceLevel(AIRCTRL_AILERON,0,true);
		if(!(bFairingAttached && status==2))bManualSeparate = true;
		timer = jettison_time;
		//autop = 0;
	}
	else if (status == 2 && GetThrusterLevel(th_main[0]) < 1 && timer > 0)
	{
		if (jettison_time-timer > 10)
			{
				timer = 0;	
				SetThrusterLevel(th_main[0],1);
			}
	}

	else if (status == 2 && GetThrusterLevel(th_main[0]) > 0)
	{
		double tot_mass = GetMass();
		double fuel_mass = GetPropellantMass(ph_stage2);
		double fuel_isp = GetThrusterIsp(th_main[0]);
		double thrust = GetThrusterMax(th_main[0]);
		double rem_burn_time = fuel_mass*fuel_isp/thrust;
		double final_mass = tot_mass-fuel_mass;
		double current_acc = thrust/tot_mass;
		double final_acc = thrust/final_mass;

		double rate_factor = final_acc/current_acc;
		
		double tgt_ap = 1000*tgt_api;
		tgt_ap = tgt_ap+6371010-2000;
		double current_alt = GetAltitude();
		//if (current_alt > 113000 && bFairingAttached)SeparateFairings();
		if ((fuel_mass/GetPropellantMaxMass(ph_stage2)) < 0.33  && bFairingAttached)SeparateFairings();
		VECTOR3 vvel;
		GetHorizonAirspeedVector(vvel);

	OBJHANDLE hRef = GetGravityRef();
	ELEMENTS el;
	ORBITPARAM prm;
	GetElements (hRef,el,&prm);
	

		double ap_alt = prm.ApD;
		double dtApD = rate_factor*(ap_alt - ap_alt_old)/oapiGetSimStep();

		double ap_predict = (dtApD*rem_burn_time)+ap_alt;
		if (rem_burn_time < 0.2)PitchStop();
		else if (ap_predict < (tgt_ap-2000)) PitchUp();
		else if (ap_predict > (tgt_ap+2000)) PitchDown();
		else if (ap_predict < (tgt_ap+500) && ap_predict > (tgt_ap-500))PitchStop();
		else SetThrusterDir(th_main[0], _V(0,0,1));

		//sprintf(oapiDebugString(),"tgt %0.1f",ap_predict-tgt_ap);

		ap_alt_old = ap_alt;

		if(bFairingAttached == false)HeadsUp();
	}

	else if (stg_sw == 1)
	{
		SetThrusterDir(th_main[0], _V(0,0,1));
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
		bManualSeparate = true;
		//autop = 0;
		//sprintf(oapiDebugString(),"jet stg 2");
		stg_sw = 0;
	}
	else if (status == 3)
	{
	if (GetPropellantMass(ph_stage3) < 1)
		{	
		SetThrusterDir(th_main[0], _V(0,0,1));
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
		ActivateNavmode(NAVMODE_KILLROT);
		autop = 2;
		}
	else if(GetThrusterLevel(th_main[0]) < 1)
		{
			OBJHANDLE hRef = GetGravityRef();
			ELEMENTS el;
			ORBITPARAM prm;
			GetElements (hRef,el,&prm);

			double t2ap = 0;
			if (prm.ApT > prm.PeT) t2ap = 0;
			else t2ap = prm.ApT;

		double fuel_mass = GetPropellantMass(ph_stage3);
		double fuel_isp = GetThrusterIsp(th_main[0]);
		double thrust = GetThrusterMax(th_main[0]);
		double rem_burn_time = fuel_mass*fuel_isp/thrust;

		time_to_ign = t2ap-(0.5*rem_burn_time);

		if (t2ap < 0.5*rem_burn_time && GetThrusterLevel(th_main[0]) < 1)
			{
			SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
			SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
			SetThrusterLevel(th_main[0],1);
			}
		else if (GetThrusterLevel(th_main[0]) < 1)DoPrograde();
		HeadsUp();

		}

	else if  (GetThrusterLevel(th_main[0]) > 0)
	{
		double tot_mass = GetMass();
		double fuel_mass = GetPropellantMass(ph_stage3);
		double fuel_isp = GetThrusterIsp(th_main[0]);
		double thrust = GetThrusterMax(th_main[0]);
		double rem_burn_time = fuel_mass*fuel_isp/thrust;
		double final_mass = tot_mass-fuel_mass;
		double current_acc = thrust/tot_mass;
		double final_acc = thrust/final_mass;

		//double rate_factor = current_acc/final_acc;
		
		VECTOR3 vvel;
		GetHorizonAirspeedVector(vvel);
	
		double vvely = vvel.y;
		double v_acc = (vvely - vvely_old)/sim_step;

		double fctr = (90-rem_burn_time)/90;
		double v_acc_req = -(vvely-tgt_vvely)/rem_burn_time;

		if (v_acc < v_acc_req-1-fctr) PitchUp();
		else if(v_acc > v_acc_req+1-fctr) PitchDown();
		else PitchStop();

		HeadsUp();
		vvely_old = vvely;
		}
	}
}

void PegasusXL::PitchUp()
{
for (int i=1;i<8; i++) DeactivateNavmode(i);

double mass = GetMass();
double thrust = GetThrusterMax(th_main[0]);

VECTOR3 eng_pos;
GetThrusterRef(th_main[0],eng_pos);

VECTOR3 vel;
GetHorizonAirspeedVector(vel);
VECTOR3 my_pmi;
GetPMI(my_pmi);
double rot_inertx = mass*my_pmi.x;
double rot_inertz = mass*my_pmi.z;
VECTOR3 avel;
GetAngularVel(avel);
double max_torque = 0.017452*thrust*eng_pos.z;
double max_avel = 1*RAD;
double th_level = 0;
double max_ang_acc = max_torque/rot_inertx;
double tf = 0.5;

if (avel.x > max_avel)
	{
	th_level = tf*(avel.x - max_avel)/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,-eng_y_comp,eng_z_comp));
	}

else if (avel.x < max_avel)
	{
	th_level = tf*(max_avel - avel.x)/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,eng_y_comp,eng_z_comp));
	}


}

void PegasusXL::PitchDown()
{
for (int i=1;i<8; i++) DeactivateNavmode(i);

double mass = GetMass();
double thrust = GetThrusterMax(th_main[0]);

VECTOR3 eng_pos;
GetThrusterRef(th_main[0],eng_pos);

VECTOR3 vel;
GetHorizonAirspeedVector(vel);
VECTOR3 my_pmi;
GetPMI(my_pmi);
double rot_inertx = mass*my_pmi.x;
double rot_inertz = mass*my_pmi.z;
VECTOR3 avel;
GetAngularVel(avel);
double max_torque = 0.017452*thrust*eng_pos.z;
double max_avel = 1*RAD;
double th_level = 0;
double max_ang_acc = max_torque/rot_inertx;
double tf = 0.5;

if (avel.x > -max_avel)
	{
	th_level = tf*(max_avel + avel.x)/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,-eng_y_comp,eng_z_comp));
	//sprintf(oapiDebugString(),"eng y z DN %0.4f %0.4f ",eng_y_comp,eng_z_comp);
	}

else if (avel.x < -max_avel)
	{
	th_level = tf*(-max_avel - avel.x)/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,eng_y_comp,eng_z_comp));
	//sprintf(oapiDebugString(),"eng y z UP %0.4f %0.4f ",eng_y_comp,eng_z_comp);
	}


}

void PegasusXL::PitchStop()
{
for (int i=1;i<8; i++) DeactivateNavmode(i);

double mass = GetMass();
double thrust = GetThrusterMax(th_main[0]);

VECTOR3 eng_pos;
GetThrusterRef(th_main[0],eng_pos);

VECTOR3 vel;
GetHorizonAirspeedVector(vel);
VECTOR3 my_pmi;
GetPMI(my_pmi);
double rot_inertx = mass*my_pmi.x;
double rot_inertz = mass*my_pmi.z;
VECTOR3 avel;
GetAngularVel(avel);
double max_torque = 0.017452*thrust*eng_pos.z;
double max_avel = 5*RAD;
double th_level = 0;
double max_ang_acc = max_torque/rot_inertx;


if (avel.x > 0)
	{
	th_level = avel.x/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,-eng_y_comp,eng_z_comp));
	}

else if (avel.x < 0)
	{
	th_level =  -avel.x/(max_ang_acc*sim_step);
	if (th_level > 1) th_level = 1;
	double eng_y_comp = th_level*0.017452;
	double eng_z_comp = sqrt(1-(eng_y_comp*eng_y_comp));
	SetThrusterDir(th_main[0], _V(0,eng_y_comp,eng_z_comp));
	}


}

void PegasusXL::HeadsUp()
{
double mass = GetMass();

double bank_angle = GetBank();

VECTOR3 my_pmi;
GetPMI(my_pmi);

double rot_inertz = mass*my_pmi.z;
VECTOR3 avel;
GetAngularVel(avel);

double max_torque = 40;
double max_avel = 2*RAD;
double th_level = 0;
double max_ang_acc = max_torque/rot_inertz;

double min_tts = avel.z/max_torque;
double tthu = bank_angle/avel.z;

double req_avelz = bank_angle*2;
if (req_avelz > max_avel) req_avelz = max_avel;
else if(req_avelz < -max_avel) req_avelz = -max_avel;

double avel_diff = req_avelz-avel.z;

double req_acc = avel_diff/sim_step;

th_level = req_acc/max_ang_acc;

if (th_level < 0) th_level = -th_level;
if (th_level > 1) th_level = 1;

if (req_acc > 0)
{
	SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,th_level);
	SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
}

else if (req_acc < 0)
{
	SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
	SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,th_level);
}


}

void PegasusXL::DoPrograde()
{
double mass = GetMass();

double aoa_angle = GetAOA();

VECTOR3 my_pmi;
GetPMI(my_pmi);

double rot_inertx = mass*my_pmi.x;
VECTOR3 avel;
GetAngularVel(avel);

double max_torque = 40;
double max_avel = 2*RAD;
double th_level = 0;
double max_ang_acc = max_torque/rot_inertx;

double min_tts = avel.x/max_torque;
double tthu = aoa_angle/avel.x;

double req_avelx = -aoa_angle*2;
if (req_avelx > max_avel) req_avelx = max_avel;
else if(req_avelx < -max_avel) req_avelx = -max_avel;

double avel_diff = req_avelx-avel.x;

double req_acc = avel_diff/sim_step;

th_level = req_acc/max_ang_acc;

if (th_level < 0) th_level = -th_level;
if (th_level > 1) th_level = 1;

if (req_acc > 0)
{
	SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,th_level);
	SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
}

else if (req_acc < 0)
{
	SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
	SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,th_level);
}


}

void PegasusXL::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}

	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);

	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	if (autop == 0)
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"M = Ascent Autopilot On/Off : Off");
	TextOut(hDC,(10+roxl), (wd+royl), kbuf, strlen(kbuf));
	}
	else if (autop == 1)
	{
	SetTextColor(hDC,YELLOW);
	char kbuf[256];
	sprintf(kbuf,"M = Ascent Autopilot On/Off : On");
	TextOut(hDC,(10+roxl), (wd+royl), kbuf, strlen(kbuf));
	}
	else if (autop == 2)
	{
	SetTextColor(hDC,YELLOW);
	char kbuf[256];
	sprintf(kbuf,"Ascent Autopilot Finished");
	TextOut(hDC,(10+roxl), (wd+royl), kbuf, strlen(kbuf));
	}

	if (autop == 0 && status < 3)
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"B = Stg.2 Apogee Tgt. %0.1f km", tgt_api);
	TextOut(hDC,(10+roxl), (wc+royl), kbuf, strlen(kbuf));
	}
	else if (autop == 1 && status < 3)
	{
	SetTextColor(hDC,YELLOW);
	char kbuf[256];
	sprintf(kbuf,"B = Stg.2 Apogee Tgt. %0.1f km", tgt_api);
	TextOut(hDC,(10+roxl), (wc+royl), kbuf, strlen(kbuf));
	}

	if (status == 3 && autop == 1)
	{
	SetTextColor(hDC,YELLOW);
	char kbuf[256];
	sprintf(kbuf,"Time to Stg.3 Burn %d sec", ((int) (time_to_ign)));
	TextOut(hDC,(10+roxl), (wc+royl), kbuf, strlen(kbuf));
	}

	if (jettison_time == 0)
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"N = Release from aircraft");
	TextOut(hDC,(10+roxl), (w2+royl), kbuf, strlen(kbuf));
	}
	if (status < 3)
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"J = Jettison Stage");
	TextOut(hDC,(10+roxl), (w3+royl), kbuf, strlen(kbuf));
	}
	else
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"J = Jettison Payload");
	TextOut(hDC,(10+roxl), (w3+royl), kbuf, strlen(kbuf));
	}

	if (bFairingAttached)
	{
	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"K = Jettison Fairing");
	TextOut(hDC,(10+roxl), (w4+royl), kbuf, strlen(kbuf));
	}

	if (status == 3)
	{
	double tot_mass = GetMass();
	double fuel_mass = GetPropellantMass(ph_stage3);
	double fuel_isp = GetThrusterIsp(th_main[0]);

	double mass_fraction = ((tot_mass)/(tot_mass-fuel_mass));
	double dvr = (fuel_isp*(log(mass_fraction)));

	SetTextColor(hDC,GREEN);
	char kbuf[256];
	sprintf(kbuf,"Stg3 dV = %0.2f m/s", dvr);
	TextOut(hDC,(10+roxl), (w1+royl), kbuf, strlen(kbuf));
	}
}